// HPModel.h

#ifndef HPMODEL_H
#define HPMODEL_H

#include <iostream>
#include <vector>
#include <string>
#include <stdexcept>

// Enum for relative direction of folding
enum DIRECTION {
    FORWARD,
    UP,
    DOWN,
    LEFT,
    RIGHT
};

// Enum for absolute direction in 3D space
enum ABSOLUTE_DIRECTION {
    NORTH,
    EAST,
    SOUTH,
    WEST,
    ABS_UP,
    ABS_DOWN
};

// Enum for rotation state
enum ROTATION {
    NO_ROTATION,
    ROTATE_90,
    ROTATE_180,
    ROTATE_270
};

// Class representing a point in 3D space
class Point {
public:
    int x;
    int y;
    int z;
    bool hydrophobic;

    Point(const int x, const int y, const int z, const bool hydrophobic);
    void print() const;
    bool coords_match(Point other) const;
};

// HPModel class representing the HP protein folding model
class HPModel {
private:
    std::vector<Point> acids;          // The list of amino acids in the chain
    std::string sequence;              // The HP sequence (e.g., "hphph")

    ABSOLUTE_DIRECTION currentDirection;
    ABSOLUTE_DIRECTION previousDirection;
    ROTATION rotation;

    void updateCurrentDirection(DIRECTION direction);
    ABSOLUTE_DIRECTION getPlaneDirectionChange(ABSOLUTE_DIRECTION absolute_direction, DIRECTION relative_direction);
    ABSOLUTE_DIRECTION getUpDownDirectionAndUpdatePrevious(DIRECTION relative_direction);
    bool isInsertValid(Point toInsert);
    Point getHead();
    bool isCurrentAcidHydrophobic();
    void addPoint(DIRECTION direction, bool isHydrophobic);
    DIRECTION getRotatedRelativeDirection(DIRECTION relative_direction);
    ABSOLUTE_DIRECTION invertDirection(ABSOLUTE_DIRECTION direction);
    void rotate(int amount);

public:
    HPModel(const std::string &input_sequence);

    void fold(DIRECTION direction);
    void print() const;
    std::vector<Point> getPoints();
    std::string pointsToJson() const;
    static std::string directionToString(DIRECTION direction);
};

#endif // HPMODEL_H