//
// Created by William on 23/10/2024.
//

#include <utility>
#include <vector>
#include <sstream>
#include "LongTermCacheData.h"


std::string LongTermCacheData::toString() {
    std::string impossiblePrefixesString;
    std::string strPrefix;
    for (const auto& prefix : impossiblePrefixes) {
        impossiblePrefixesString += strPrefix;
        impossiblePrefixesString += prefix;
        strPrefix = ",";
    }

    std::string checkedFoldsString;
    strPrefix = "";
    for (const auto& fold : checkedFolds) {
        checkedFoldsString += strPrefix;
        checkedFoldsString += fold;
        strPrefix = ",";
    }
    std::string bestScoreString = std::to_string(bestScore);
    std::string currentFoldLengthString = std::to_string(currentFoldLength);
    return impossiblePrefixesString + ';' + checkedFoldsString + ';' + bestFold + ';' + bestScoreString + ';' + currentFoldLengthString;
}

LongTermCacheData::LongTermCacheData() {
    bestFold = "";
    bestScore = 0;
    currentFoldLength = 0;
}

LongTermCacheData::LongTermCacheData(std::vector<std::string> impossible, std::vector<std::string> checked,
                                     std::string fold, int score, int length) : impossiblePrefixes(
        std::move(impossible)),
                                                                                checkedFolds(std::move(checked)),
                                                                                bestFold(std::move(fold)),
                                                                                bestScore(score),
                                                                                currentFoldLength(length) {}

bool LongTermCacheData::isEmpty() {
    return bestFold.empty() && bestScore == 0 && currentFoldLength == 0;
}



std::vector<std::string> splitString(std::string toSplit, char delim) {
    std::vector<std::string> res;
    std::string temp;
    std::stringstream ss(toSplit);
    while (std::getline(ss, toSplit, delim)) {
        res.push_back(temp);
    }

    return res;
}

LongTermCacheData fromString(const std::string &string) {
    if (string == "NONE") {
        return {};
    }

    auto split = splitString(string, ';');

    std::vector<std::string> impossiblePrefixes{};
    std::string impossibleString = split[0];
    auto impossibleSplit = splitString(impossibleString, ',');
    impossiblePrefixes.reserve(impossibleSplit.size());
    for (const auto &prefix: impossibleSplit) {
        impossiblePrefixes.push_back(prefix);
    }

    std::vector<std::string> checkedFolds{};
    std::string checkedFoldsString = split[1];
    auto checkedFoldsSplit = splitString(checkedFoldsString, ',');
    checkedFolds.reserve(checkedFoldsSplit.size());
    for (const auto &checkedFold : checkedFoldsSplit) {
        checkedFolds.push_back(checkedFold);
    }

    std::string bestFold = split[2];
    int bestScore = std::stoi(split[3]);
    int currentFoldLength = std::stoi(split[4]);


    return LongTermCacheData{
            impossiblePrefixes,
            checkedFolds,
            bestFold,
            bestScore,
            currentFoldLength
    };
}
void LongTermCacheData::fromDataString(const std::string &dataString) {
    auto data = fromString(dataString);
    this->impossiblePrefixes = data.impossiblePrefixes;
    this->checkedFolds = data.checkedFolds;
    this->bestFold = data.bestFold;
    this->bestScore = data.bestScore;
    this->currentFoldLength = data.currentFoldLength;
}
